<?php

namespace Drupal\masquerade_as_role\Form;

use Drupal\Core\Form\FormStateInterface;
use Drupal\masquerade\Form\MasqueradeForm;

/**
 * Form builder for the masquerade as role form.
 */
class MasqueradeAsRoleForm extends MasqueradeForm {

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * The masquerade service.
   *
   * @var \Drupal\masquerade\Masquerade
   */
  protected $masquerade;

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    // @todo Change the autogenerated stub.
    $form = parent::buildForm($form, $form_state);
    $form['autocomplete']['masquerade_as']['#required'] = FALSE;
    $roles = user_roles();
    foreach ($roles as $key => $role) {
      // No need to masquerade using anonymous role.
      if ($key !== 'anonymous') {
        $roleOptions[$key] = $role->label();
      }
    }
    $form['autocomplete']['masquerade_as']['#description'] = $this->t('Enter the username to masquerade as.<br>');
    $form['autocomplete']['masquerade_as_role'] = [
      '#type' => 'select',
      '#options' => $roleOptions,
      '#description' => $this->t('Alternatively, select one or several role(s) to masquerade as.<br>'),
      '#multiple' => TRUE,
    ];
    // Set appropriate weights.
    $form['autocomplete']['masquerade_as']['#weight'] = 1;
    $form['autocomplete']['masquerade_as_role']['#weight'] = 2;
    $form['submit']['#weight'] = 3;
    $form['#attached']['library'][] = 'masquerade_as_role/masquerade_as_role';

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state) {
    $rids = $form_state->getValue('masquerade_as_role');
    if (empty($rids)) {
      $name = $form_state->getValue('masquerade_as');
      if (empty($name)) {
        $form_state->setErrorByName('masquerade_as', $this->t('The user does not exist. Please enter a valid username.'));
        return;
      }
      else {
        return parent::validateForm($form, $form_state);
      }
    }
    $edit = [];
    $edit['name'] = substr(md5(microtime()), rand(0, 26), 8);
    $edit['mail'] = $edit['name'] . 'masquerade_as_role@example.com';
    $edit['pass'] = \Drupal::service('password_generator')->generate();
    $edit['roles'] = array_keys($rids);
    $edit['status'] = 1;
    $target_account = $this->entityTypeManager
      ->getStorage('user')->create($edit);
    $target_account->save();
    if ($error = masquerade_switch_user_validate($target_account)) {
      $form_state->setErrorByName('masquerade_as', $error);
    }
    else {
      $form_state->setValue('masquerade_target_account', $target_account);
    }
  }

}
